"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkOrDownloadLatestVersion = exports.DownloadError = void 0;
const messages = require("./Messages");
const fs = require("fs-extra");
const path = require("path");
const stream = require("stream");
const util = require("util");
const yauzl = require("yauzl-promise");
const node_fetch_1 = require("node-fetch");
const buffer_1 = require("buffer");
const Util_1 = require("./Util");
const VersionManager_1 = require("./VersionManager");
const streamPipeline = util.promisify(stream.pipeline);
const endpoint = "/api/query/Users/DownloadSwarm";
class DownloadError extends Error {
}
exports.DownloadError = DownloadError;
async function checkOrDownloadLatestVersion(url, lastVersion, targetDir, platform, runFileName) {
    if (!lastVersion) {
        throw new DownloadError((0, Util_1.formatString)(messages.requestfailedWithCode));
    }
    var endpointUrl = buildEnpointQuery(url, lastVersion, platform);
    var response = await (0, node_fetch_1.default)(endpointUrl);
    var version = response.headers.get("swarm-version");
    if (response.status === 200) {
        console.info(messages.downloadingAndUnzipping);
        var unzipDir = path.join(targetDir, Date.now().toString());
        var targetReleaseDir = path.join(targetDir, version);
        var responseBuffer = await response.arrayBuffer();
        await unzip(buffer_1.Buffer.from(responseBuffer), unzipDir);
        try {
            await copySwarmFiles(unzipDir, targetReleaseDir, runFileName);
        }
        finally {
            await fs.remove(unzipDir);
        }
        return new VersionManager_1.LastVersion(version, url, new Date);
    }
    if (response.status === 204) {
        return new VersionManager_1.LastVersion(lastVersion !== null && lastVersion !== void 0 ? lastVersion : version, url, new Date());
    }
    throw new DownloadError((0, Util_1.formatString)(messages.requestfailedWithCode, response.status.toString()));
}
exports.checkOrDownloadLatestVersion = checkOrDownloadLatestVersion;
async function unzip(buffer, targetDir) {
    var zipfile = await yauzl.fromBuffer(buffer, { lazyEntries: true });
    var entry;
    while (entry = await zipfile.readEntry()) {
        if (entry.compressedSize === 0) {
            continue;
        }
        await fs.mkdirp(path.join(targetDir, path.dirname(entry.fileName)));
        var readStream = await entry.openReadStream();
        var writeStream = fs.createWriteStream(path.join(targetDir, entry.fileName));
        await streamPipeline(readStream, writeStream);
    }
}
async function copySwarmFiles(sourceDir, targetDir, runFileName) {
    await fs.copy(sourceDir, targetDir);
    const runFullName = path.join(targetDir, runFileName);
    if (await fs.pathExists(runFullName)) {
        var stat = await fs.stat(runFullName);
        await fs.chmod(runFullName, stat.mode | 0o110);
    }
}
function buildEnpointQuery(baseUrl, version, platform) {
    var resultUrl = baseUrl.concat(endpoint);
    resultUrl = resultUrl.concat("?Version=", version);
    resultUrl = resultUrl.concat("&Platform=", platform);
    return resultUrl;
}
