"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs-extra");
const os = require("os");
const stream = require("stream");
const util = require("util");
const yauzl = require("yauzl-promise");
const streamPipeline = util.promisify(stream.pipeline);
const creatingParentDir = "Creating parent directory: '{0}'.";
const cannotCreateParentDir = "Cannot create parent directory: '{0}'.";
const unsupportedPlatform = "No version found for the current platform: '{0}'.";
const couldNotInstall = "Could not install default version of swarm.";
const url = "https://quadientcloud.eu";
const platformToAssetNameMap = {
    "win32-x64": "win_x64",
    "win32-x86": "win_x64",
    "win32-x32": "win_x64",
    "win32-ia32": "win_x64",
    "linux-x64": "linux_x64",
    "darwin-x64": "osx_x64",
    "darwin-arm64": "osx_arm64",
};
const platformWithArch = os.platform() + "-" + os.arch();
const platform = platformToAssetNameMap[platformWithArch] || platformWithArch;
const runFile = "swarm";
const homeDir = path.join(os.homedir(), ".swarm");
const lastVersionFileName = path.join(homeDir, ".lastversion");
main();
async function main() {
    try {
        await work();
    }
    catch (error) {
        console.error(error.message);
        console.error(couldNotInstall);
    }
}
async function work() {
    await checkOrCreateDir(homeDir);
    const release = path.join(__dirname, "..", "release");
    if (!(await fs.pathExists(release))) {
        return;
    }
    var zipName = await getZipName(release);
    if (!zipName) {
        return;
    }
    var version = zipName.replace("Swarm-", "").replace(".zip", "").trim();
    var unzipDir = path.join(release, Date.now().toString());
    await unzip(path.join(release, zipName), unzipDir);
    await copyFilesAccordingToPlatform(unzipDir, path.join(homeDir, version), platform, runFile);
    await fs.remove(unzipDir);
    await fs.writeFile(lastVersionFileName, JSON.stringify({
        version: version,
        cloudUrl: url,
        timestamp: new Date(1)
    }));
}
async function checkOrCreateDir(dir) {
    if (!(await fs.pathExists(dir))) {
        try {
            console.log(formatString(creatingParentDir, dir));
            await fs.mkdirp(dir);
        }
        catch {
            throw (new Error(formatString(cannotCreateParentDir, dir)));
        }
    }
}
async function getZipName(dirPath) {
    var files = await fs.readdir(dirPath);
    for (var file of files) {
        if (file.startsWith("Swarm-") && file.endsWith(".zip")) {
            return file;
        }
    }
    return null;
}
async function unzip(file, targetDir) {
    var zipfile = await yauzl.open(file, { lazyEntries: true });
    var entry;
    while (entry = await zipfile.readEntry()) {
        if (entry.compressedSize === 0) {
            continue;
        }
        await fs.mkdirp(path.join(targetDir, path.dirname(entry.fileName)));
        var readStream = await entry.openReadStream();
        var writeStream = fs.createWriteStream(path.join(targetDir, entry.fileName));
        await streamPipeline(readStream, writeStream);
    }
}
async function copyFilesAccordingToPlatform(sourceDir, targetDir, platform, runFileName) {
    var platformDir = await findSubdirFullPath(sourceDir, platform);
    if (!platformDir) {
        throw Error(formatString(unsupportedPlatform, platform));
    }
    await fs.copy(platformDir, targetDir);
    let runFullName = path.join(targetDir, runFileName);
    if (await fs.pathExists(runFullName)) {
        var stat = await fs.stat(runFullName);
        await fs.chmod(runFullName, stat.mode | 0o110);
    }
}
async function findSubdirFullPath(dirPath, subdirName) {
    var files = await fs.readdir(dirPath);
    for (var file of files) {
        var fullPath = path.join(dirPath, file);
        if (file === subdirName) {
            return fullPath;
        }
        if ((await fs.stat(fullPath)).isDirectory()) {
            var result = await findSubdirFullPath(fullPath, subdirName);
            if (result) {
                return result;
            }
        }
    }
    return null;
}
function formatString(template, ...args) {
    for (let index = 0; index < args.length; index++) {
        template = template.replace(`{${index}}`, args[index]);
    }
    return template;
}
